/*
--------------------
Program: ESQUI
Info and Bugs: {marf,rsantana}@ctm.ulpgc.es 
-------------------------------------------

Copyright (c) 2006, Center for Technology in Medicine, University of Las Palmas
de Gran Canaria (Canary Islands, Spain).
All rights reserved.

This program is free software; you can redistribute it and/or modify it under 
the terms of the Creative Commons' "by-attribution" license 
(http://creativecommons.org/licenses/by/2.5).

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1) Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2) Redistributions in binary form must reproduce the above copyright 
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.


THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
=============================================================================*/
#ifndef __vtkOrganC_h
#define __vtkOrganC_h

#include "vtkCollection.h"
#include "vtkesquiT2MeshWin32Header.h"
class vtkOrgan;

//!  A list of deformable organs
/*! 
   vtkOrganCollection represents and provides methods to manipulate a list of
   Organs (i.e., vtkOrgan and subclasses). The list is unsorted and duplicate
   entries are not prevented.
   \sa vtkCollection, vtkOrgan
*/
class VTK_ESQUI_T2MESH_EXPORT vtkOrganCollection : public vtkCollection
{
 public:
  static vtkOrganCollection *New();
  vtkTypeRevisionMacro(vtkOrganCollection,vtkCollection);
  virtual void PrintSelf(ostream& os, vtkIndent indent);

  //! Add a Organ to the list.
  void InsertOrgan(vtkOrgan *a);
  
  //! Get a Organ in the list.
  /*
     Get the organ in the list with the position i. NULL is returned when the collection is 
     exhausted.
  */
  void InsertOrganPosition(int i, vtkOrgan *a);
  vtkOrgan *GetModelOnPosition(int i);

protected:
  vtkOrganCollection() {};
  ~vtkOrganCollection() {};


private:
  /// hide the standard ReplaceItem from the user and the compiler.
  void AddItem(vtkObject *o) { this->vtkCollection::AddItem(o); }

private:
  vtkOrganCollection(const vtkOrganCollection&);  // Not implemented.
  void operator=(const vtkOrganCollection&);					// Not implemented.
};


#endif
